@@ -31,6 +31,7 @@
 #include "fio.h"
 #include "smalloc.h"
 #include "os/os.h"
+#include "filelock.h"
 
 /*
  * Just expose an empty list, if the OS does not support disk util stats
@@ -135,7 +136,7 @@ void reset_all_stats(struct thread_data *td)
 	memcpy(&td->epoch, &tv, sizeof(tv));
 	memcpy(&td->start, &tv, sizeof(tv));
 
-	lat_target_init(td);
+	lat_target_reset(td);
 }
 
 void reset_fio_state(void)
@@ -172,6 +173,19 @@ void td_set_runstate(struct thread_data *td, int runstate)
 	td->runstate = runstate;
 }
 
+int td_bump_runstate(struct thread_data *td, int new_state)
+{
+	int old_state = td->runstate;
+
+	td_set_runstate(td, new_state);
+	return old_state;
+}
+
+void td_restore_runstate(struct thread_data *td, int old_state)
+{
+	td_set_runstate(td, old_state);
+}
+
 void fio_terminate_threads(int group_id)
 {
 	struct thread_data *td;
@@ -204,6 +218,21 @@ void fio_terminate_threads(int group_id)
 	}
 }
 
+int fio_running_or_pending_io_threads(void)
+{
+	struct thread_data *td;
+	int i;
+
+	for_each_td(td, i) {
+		if (td->flags & TD_F_NOIO)
+			continue;
+		if (td->runstate < TD_EXITED)
+			return 1;
+	}
+
+	return 0;
+}
+
 static int endian_check(void)
 {
 	union {
@@ -252,6 +281,11 @@ int initialize_fio(char *envp[])
 
 	sinit();
 
+	if (fio_filelock_init()) {
+		log_err("fio: failed initializing filelock subsys\n");
+		return 1;
+	}
+
 	/*
 	 * We need locale for number printing, if it isn't set then just
 	 * go with the US format.
