@@ -122,21 +122,48 @@ static void show_option_help(struct fio_option *o, int is_err)
 	show_option_values(o);
 }
 
-static unsigned long get_mult_time(char c)
+static unsigned long long get_mult_time(const char *str, int len,
+					int is_seconds)
 {
-	switch (c) {
-	case 'm':
-	case 'M':
-		return 60;
-	case 'h':
-	case 'H':
-		return 60 * 60;
-	case 'd':
-	case 'D':
-		return 24 * 60 * 60;
-	default:
-		return 1;
+	const char *p = str;
+	char *c;
+	unsigned long long mult = 1;
+
+	/*
+         * Go forward until we hit a non-digit, or +/- sign
+         */
+	while ((p - str) <= len) {
+		if (!isdigit((int) *p) && (*p != '+') && (*p != '-'))
+			break;
+		p++;
 	}
+
+	if (!isalpha((int) *p)) {
+		if (is_seconds)
+			return 1000000UL;
+		else
+			return 1;
+	}
+
+	c = strdup(p);
+	for (int i = 0; i < strlen(c); i++)
+		c[i] = tolower(c[i]);
+
+	if (!strncmp("us", c, 2) || !strncmp("usec", c, 4))
+		mult = 1;
+	else if (!strncmp("ms", c, 2) || !strncmp("msec", c, 4))
+		mult = 1000;
+	else if (!strcmp("s", c))
+		mult = 1000000;
+	else if (!strcmp("m", c))
+		mult = 60 * 1000000UL;
+	else if (!strcmp("h", c))
+		mult = 60 * 60 * 1000000UL;
+	else if (!strcmp("d", c))
+		mult = 24 * 60 * 60 * 1000000UL;
+
+	free(c);
+	return mult;
 }
 
 static int is_separator(char c)
@@ -248,7 +275,8 @@ int str_to_float(const char *str, double *val)
 /*
  * convert string into decimal value, noting any size suffix
  */
-int str_to_decimal(const char *str, long long *val, int kilo, void *data)
+int str_to_decimal(const char *str, long long *val, int kilo, void *data,
+		   int is_seconds)
 {
 	int len, base;
 
@@ -275,19 +303,19 @@ int str_to_decimal(const char *str, long long *val, int kilo, void *data)
 		else
 			*val *= mult;
 	} else
-		*val *= get_mult_time(str[len - 1]);
+		*val *= get_mult_time(str, len, is_seconds);
 
 	return 0;
 }
 
 int check_str_bytes(const char *p, long long *val, void *data)
 {
-	return str_to_decimal(p, val, 1, data);
+	return str_to_decimal(p, val, 1, data, 0);
 }
 
-int check_str_time(const char *p, long long *val)
+int check_str_time(const char *p, long long *val, int is_seconds)
 {
-	return str_to_decimal(p, val, 0, NULL);
+	return str_to_decimal(p, val, 0, NULL, is_seconds);
 }
 
 void strip_blank_front(char **p)
@@ -329,7 +357,7 @@ static int check_range_bytes(const char *str, long *val, void *data)
 {
 	long long __val;
 
-	if (!str_to_decimal(str, &__val, 1, data)) {
+	if (!str_to_decimal(str, &__val, 1, data, 0)) {
 		*val = __val;
 		return 0;
 	}
@@ -368,16 +396,13 @@ static int str_match_len(const struct value_pair *vp, const char *str)
 	return max(strlen(vp->ival), opt_len(str));
 }
 
-#define val_store(ptr, val, off, or, data, o)			\
-	do {							\
-		if ((o)->prof_opts)				\
-			ptr = td_var((o)->prof_opts, (off));	\
-		else						\
-			ptr = td_var((data), (off));		\
-		if ((or))					\
-			*ptr |= (val);				\
-		else						\
-			*ptr = (val);				\
+#define val_store(ptr, val, off, or, data, o)		\
+	do {						\
+		ptr = td_var((data), (o), (off));	\
+		if ((or))				\
+			*ptr |= (val);			\
+		else					\
+			*ptr = (val);			\
 	} while (0)
 
 static int __handle_option(struct fio_option *o, const char *ptr, void *data,
@@ -418,7 +443,7 @@ static int __handle_option(struct fio_option *o, const char *ptr, void *data,
 			if (!strncmp(vp->ival, ptr, str_match_len(vp, ptr))) {
 				ret = 0;
 				if (o->off1)
-					val_store(ilp, vp->oval, o->off1, vp->or, data, o);
+					val_store(ilp, vp->oval, o->off1, vp->orval, data, o);
 				continue;
 			}
 		}
@@ -442,7 +467,7 @@ static int __handle_option(struct fio_option *o, const char *ptr, void *data,
 			*p = '\0';
 
 		if (is_time)
-			ret = check_str_time(tmp, &ull);
+			ret = check_str_time(tmp, &ull, o->is_seconds);
 		else
 			ret = check_str_bytes(tmp, &ull, data);
 
@@ -526,17 +551,11 @@ static int __handle_option(struct fio_option *o, const char *ptr, void *data,
 			*/
 			if (o->off2) {
 				ul2 = 0;
-				if (o->prof_opts)
-					ilp = td_var(o->prof_opts, o->off2);
-				else
-					ilp = td_var(data, o->off2);
+				ilp = td_var(data, o, o->off2);
 				*ilp = ul2;
 			}
 
-			if (o->prof_opts)
-				flp = td_var(o->prof_opts, o->off1);
-			else
-				flp = td_var(data, o->off1);
+			flp = td_var(data, o, o->off1);
 			for(i = 0; i < o->maxlen; i++)
 				flp[i].u.f = 0.0;
 		}
@@ -560,10 +579,7 @@ static int __handle_option(struct fio_option *o, const char *ptr, void *data,
 			return 1;
 		}
 
-		if (o->prof_opts)
-			flp = td_var(o->prof_opts, o->off1);
-		else
-			flp = td_var(data, o->off1);
+		flp = td_var(data, o, o->off1);
 		flp[curr].u.f = uf;
 
 		dprint(FD_PARSE, "  out=%f\n", uf);
@@ -581,10 +597,7 @@ static int __handle_option(struct fio_option *o, const char *ptr, void *data,
 				len++;
 
 			if (o->off2) {
-				if (o->prof_opts)
-					ilp = td_var(o->prof_opts, o->off2);
-				else
-					ilp = td_var(data, o->off2);
+				ilp = td_var(data, o, o->off2);
 				if (len > *ilp)
 					*ilp = len;
 			}
@@ -595,12 +608,11 @@ static int __handle_option(struct fio_option *o, const char *ptr, void *data,
 	case FIO_OPT_STR_STORE: {
 		fio_opt_str_fn *fn = o->cb;
 
-		if (o->off1) {
-			if (o->prof_opts)
-				cp = td_var(o->prof_opts, o->off1);
-			else
-				cp = td_var(data, o->off1);
+		if (!strlen(ptr))
+			return 1;
 
+		if (o->off1) {
+			cp = td_var(data, o, o->off1);
 			*cp = strdup(ptr);
 		}
 
@@ -1201,10 +1213,7 @@ void options_free(struct fio_option *options, void *data)
 		if (o->type != FIO_OPT_STR_STORE || !o->off1)
 			continue;
 
-		if (o->prof_opts)
-			ptr = td_var(o->prof_opts, o->off1);
-		else
-			ptr = td_var(data, o->off1);
+		ptr = td_var(data, o, o->off1);
 		if (*ptr) {
 			free(*ptr);
 			*ptr = NULL;
